@@ -1,6 +1,6 @@
 /*
  *  Methods for unit-testing utiltities for cryptmount
- *  (C)Copyright 2006-2010, RW Penney
+ *  (C)Copyright 2006-2011, RW Penney
  */
 
 /*
@@ -80,6 +80,7 @@ int cm_run_tests()
     ut_test_strops();
     fs_test_blkgetsz();
     fs_test_splitopts();
+    fs_test_entropy();
     ut_test_sha1();
     ut_pwfort();
     bf_test_blowfish();
@@ -97,8 +98,6 @@ int _keep_ansi_pedantic_quiet = 0;
 
 #endif  /* TESTING */
 
-/* vim: set ts=4 sw=4 et: */
-
 /*
- *  (C)Copyright 2006-2010, RW Penney
+ *  (C)Copyright 2006-2011, RW Penney
  */
