@@ -1,6 +1,6 @@
 /*
  *  cryptmount - a utility for user-level mounting of encrypted filesystems
- *  (C)Copyright 2005-2010, RW Penney
+ *  (C)Copyright 2005-2011, RW Penney
  */
 
 /*
@@ -80,7 +80,7 @@ typedef enum                    /* Top-level operating mode */
 } cmmode_t;
 
 
-static int64_t getblkcount(const char *device);
+static int64_t getblk512count(const char *device, int *blklen);
 static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
                 const km_pw_context_t *pw_ctxt,
                 const char *params, const targelt_t *eltlist);
@@ -130,7 +130,7 @@ cm_testinfo_t test_context;
 cm_testinfo_t *test_ctxtptr=&test_context;
 
 int fs_test_blkgetsz()
-    /* Check that 32bit & 64bit device size calculations agree */
+    /** Check that 32bit & 64bit device size calculations agree */
 {
 #ifdef BLKGETSIZE64
     int fd, n_open, seclen;
@@ -142,7 +142,8 @@ int fs_test_blkgetsz()
         "/dev/hdb", "/dev/hdb1", "/dev/hdb2", "/dev/hdb3",
         "/dev/hdc", "/dev/hdc1", "/dev/hdc2", "/dev/hdc3",
         "/dev/sda", "/dev/sda1", "/dev/sda2", "/dev/sda3",
-        "/dev/scd", "/dev/scd1", "/dev/scd2", "/dev/scd3",
+        "/dev/sdb", "/dev/sdb1", "/dev/sdb2", "/dev/sdb3",
+        "/dev/sr0", "/dev/sr1",
         NULL };
 #endif
 
@@ -167,7 +168,9 @@ int fs_test_blkgetsz()
                 CM_ASSERT_EQUAL(len64, ((int64_t)len * (int64_t)512));
             }
         }
-        /* fprintf(stderr, "%s: %d  %ld  %lld\n", *dev, seclen, len * 512, len64); */
+#if 0
+        if (fd >= 0) fprintf(stderr, "%s: %d  %ld  %lld\n", *dev, seclen, len * 512, len64);
+#endif
         ++dev;
     }
 
@@ -182,30 +185,28 @@ int fs_test_blkgetsz()
 #endif  /* TESTING */
 
 
-static int64_t getblkcount(const char *device)
-    /* find size of raw device in blocks */
-{   int64_t count=-1;
+static int64_t getblk512count(const char *device, int *blklen)
+    /** Find size of raw device in blocks of size 512-bytes */
+{   int64_t count = -1;
     int fd;
-#ifdef BLKGETSIZE64
-    int blklen;
-#else
+#ifndef BLKGETSIZE64
     long len;
 #endif
 
+    *blklen = 512;
     fd = open(device, O_RDONLY);
     if (fd < 0) return (int64_t)-1;
 
 #ifdef BLKGETSIZE64
     if (ioctl(fd, BLKGETSIZE64, &count) == 0
-        && ioctl(fd, BLKSSZGET, &blklen) == 0) {
-        count /= (int64_t)blklen;
+        && ioctl(fd, BLKSSZGET, blklen) == 0) {
+        count /= (int64_t)512;
     } else {
         count = -1;
     }
 #else
     if (ioctl(fd, BLKGETSIZE, &len) == 0) {
-        /*  this is the number of 512-byte blocks,
-            which may be the best we can do without BLKGETSIZE64... */
+        /*  This directly gives the number of 512-byte blocks */
         count = (int64_t)len;
     }
 #endif
@@ -216,7 +217,7 @@ static int64_t getblkcount(const char *device)
 
 
 static int do_list(const tgtdefn_t *tgt)
-    /* Print list of available filing-system targets */
+    /** Print list of available filing-system targets */
 {
     /* TRANSLATORS: this string is marked as 'no-c-format' because
        some localizations may require the mount-point and filesystem type
@@ -232,11 +233,12 @@ static int do_list(const tgtdefn_t *tgt)
 
 static int do_devsetup(const km_pw_context_t *pw_ctxt,
                     bound_tgtdefn_t *boundtgt, char **mntdev)
-    /* Setup all devices needed to access encrypted target */
+    /** Setup all devices needed to access encrypted target */
 {   enum { BUFFMIN=1024 };
     unsigned char *key=NULL;
-    int i, readonly, isloop=0, killloop=0, keylen=0, eflag=ERR_NOERROR;
-    int64_t devlen=0, fslen=0;
+    int i, blklen, readonly, isloop = 0, killloop = 0,
+        keylen = 0, eflag = ERR_NOERROR;
+    int64_t devlen = 0, fslen = 0;
     size_t dpsize;
     char *dmparams=NULL;
     const char *tgtdev=NULL;
@@ -260,7 +262,7 @@ static int do_devsetup(const km_pw_context_t *pw_ctxt,
     }
 
     /* Get size in blocks of target device: */
-    devlen = getblkcount(tgtdev);
+    devlen = getblk512count(tgtdev, &blklen);
     if (devlen < 0) {
         fprintf(stderr, _("Failed to get size of \"%s\"\n"), tgtdev);
         eflag = ERR_BADIOCTL;
@@ -314,7 +316,7 @@ static int do_devsetup(const km_pw_context_t *pw_ctxt,
 
 
 int do_devshutdown(const bound_tgtdefn_t *boundtgt)
-    /* Remove all devices attached to encrypted target */
+    /** Remove all devices attached to encrypted target */
 {   const tgtdefn_t *tgt = boundtgt->tgt;
     struct stat sbuff;
     unsigned devcnt=0;
@@ -555,7 +557,7 @@ static int do_swapoff(const bound_tgtdefn_t *boundtgt)
 
 
 static int do_passwd(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
-    /* Change password on particular target */
+    /** Change password on particular target */
 {   tgtdefn_t *tgt = boundtgt->tgt;
     unsigned char *key=NULL;
     unsigned keyprops;
@@ -639,7 +641,7 @@ static int do_passwd(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt)
 
 static int do_keygen(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt,
                     const char *params, int reuse, const tgtdefn_t *tgttable)
-    /* Create new filesystem crypto-key */
+    /** Create new filesystem crypto-key */
 {   unsigned char *key=NULL;
     unsigned keyprops;
     int keylen=0, fileexists=0, eflag=ERR_NOERROR;
@@ -744,7 +746,7 @@ static int do_keygen(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt,
 
 
 static int do_safetynet()
-    /* Last-ditch attempt to umount/shutdown all targets currently mounted */
+    /** Last-ditch attempt to umount/shutdown all targets currently mounted */
 {   tgtstat_t *all_tsts=NULL, *tst;
     char *devname=NULL;
     const char *old_ident=NULL;
@@ -832,7 +834,7 @@ static int do_safetynet()
 
 
 static void check_priv_opt(const char *opt)
-    /* Check if ordinary user is allowed to perform privileged actions */
+    /** Check if ordinary user is allowed to perform privileged actions */
 {
 
     if (getuid() != 0) {
@@ -846,7 +848,7 @@ static void check_priv_opt(const char *opt)
 
 
 static int check_priv_tgt(const tgtdefn_t *tgt)
-    /* Check if ordinary user is allowed to perform privileged actions */
+    /** Check if ordinary user is allowed to perform privileged actions */
 {
     if ((tgt->flags & FLG_USER) == 0 && getuid() != 0) {
         fprintf(stderr, _("Only root can configure \"%s\"\n"), tgt->ident);
@@ -859,7 +861,7 @@ static int check_priv_tgt(const tgtdefn_t *tgt)
 
 static int check_mode(cmmode_t mode, const char *params,
                     const targelt_t *eltlist)
-    /* Check that arguments supplied & privilege match selected mode */
+    /** Check that arguments supplied & privilege match selected mode */
 {   enum {
         F_NORMALUSER =  0x001,      /* normal user can perform action */
         F_NEEDS_TGT =   0x010,      /* filesystem-target is required */
@@ -915,7 +917,7 @@ static int check_mode(cmmode_t mode, const char *params,
 static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
                 const km_pw_context_t *pw_ctxt,
                 const char *params, const targelt_t *eltlist)
-    /* Apply mode-dependent operation to list of targets */
+    /** Apply mode-dependent operation to list of targets */
 {   const targelt_t *elt=NULL;
     bound_tgtdefn_t *boundtgt=NULL;
     int i, prio=0, eflag=ERR_NOERROR;
@@ -1033,7 +1035,7 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
 
 
 static cmmode_t get_defaultmode(int argc, char *argv[])
-    /* Translate program-name into default action (or just assume M_MOUNT) */
+    /** Translate program-name into default action (or just assume M_MOUNT) */
 {   cmmode_t mode = M_MOUNT;
 
 #ifdef WITH_ARGV0
@@ -1291,5 +1293,5 @@ int main(int argc, char *argv[])
 }
 
 /*
- *  (C)Copyright 2005-2010, RW Penney
+ *  (C)Copyright 2005-2011, RW Penney
  */
