@@ -42,17 +42,22 @@
 #include "cryptmount.h"
 #include "dmutils.h"
 
-#ifndef UDEV_QUEUE_DIR
-#  define UDEV_QUEUE_DIR "/dev/.udev/queue"
-#endif
-#ifndef UDEV_QUEUE_FILE
-#  define UDEV_QUEUE_FILE "/dev/.udev/queue.bin"
-#endif
-int udev_queue_size();
+
+struct udev_queue_loc {
+    const char *path;
+    int is_file;
+} udev_queue_locations[] = {
+    { "/run/udev/queue.bin",    1 },    /* Debian-7.0 has queue beneath /run */
+    { "/dev/.udev/queue.bin",   1 },    /* Recent udev has file in /dev/.udev */
+    { "/dev/.udev/queue",       0 }     /* Older udev has directory of events */
+};
+
+int udev_queue_size(const char *path);
+int udev_active_dir(const char *path, time_t starttime, double timeout);
 
 
 struct dm_task *devmap_prepare(int type, const char *ident)
-    /* prepare device-mapper task structure */
+    /*! Prepare device-mapper task structure */
 {   struct dm_task *dmt=NULL;
 
     dmt = dm_task_create(type);
@@ -235,8 +240,9 @@ int is_configured(const char *ident, struct dm_info *dminfo)
 
 
 int udev_settle()
-    /* Allow time for udev events to be processed */
-{   double totdelay = 0.0, inc;
+    /*! Allow time for udev events to be processed */
+{   struct udev_queue_loc *udev_mode;
+    double totdelay = 0.0, inc;
     time_t starttime;
     struct stat sbuff;
 #if HAVE_NANOSLEEP
@@ -256,6 +262,13 @@ int udev_settle()
 
     time(&starttime);
 
+    /* Try to find location and type of udev event queue: */
+    udev_mode = udev_queue_locations;
+    while (udev_mode->is_file) {
+        if (stat(udev_mode->path, &sbuff) == 0) break;
+        ++udev_mode;
+    }
+
 #if HAVE_NANOSLEEP
     delay.tv_sec = 0.0;
     delay.tv_nsec = 2e7;
@@ -275,27 +288,27 @@ int udev_settle()
 
         settling = 0;
 
-        /* Older versions of udev place events in separate directory */
-        if (stat(UDEV_QUEUE_DIR, &sbuff) == 0) {
-            settling |= ((starttime - sbuff.st_mtime) < 100 * timeout);
+        if (udev_mode->is_file) {
+            /* Current versions of udev place events in a single file: */
+            settling |= (udev_queue_size(udev_mode->path) > 0);
+        } else {
+            /* Older versions of udev use a directory of event files: */
+            settling |= udev_active_dir(udev_mode->path, starttime, timeout);
         }
-
-        /* Newer versions of udev place events in a queue file */
-        settling |= (udev_queue_size() > 0);
     } while (settling && totdelay < timeout);
 
     return settling;
 }
 
 
-int udev_queue_size()
-    /* Count number of unprocessed udev events in queue.bin file */
+int udev_queue_size(const char *path)
+    /*! Count number of unprocessed udev events in queue.bin file */
 {   FILE *fp;
     unsigned long long seqnum;
     unsigned short skiplen;
     int nqueued = 0;
 
-    fp = fopen(UDEV_QUEUE_FILE, "rb");
+    fp = fopen(path, "rb");
     if (fp == NULL) return 0;
     if (fread((void*)&seqnum, sizeof(seqnum), (size_t)1, fp) != 1) return 0;
 
@@ -317,6 +330,26 @@ int udev_queue_size()
     return nqueued;
 }
 
+
+/*!
+ *  Check whether the udev queue directory (e.g. /dev/.udev/queue)
+ *  has been recently modified.
+ *  This is only relevant to older versions of udev (e.g. 0.105).
+ */
+int udev_active_dir(const char *path, time_t starttime, double timeout)
+{   struct stat sbuff;
+    int settling = 0;
+
+    /*  If the event directory exists, then we either have active
+        events, or possibly it is a remnant from an old udev process. */
+
+    if (stat(path, &sbuff) == 0) {
+        settling |= ((starttime - sbuff.st_mtime) < 100 * timeout);
+    }
+
+    return settling;
+}
+
 /*
  *  (C)Copyright 2005-2011, RW Penney
  */
