@@ -1,8 +1,8 @@
 /*
- * LUKS - Linux Unified Key Setup 
+ * LUKS - Linux Unified Key Setup
  *
  * Copyright (C) 2004-2006, Clemens Fruhwirth <clemens@endorphin.org>
- * 
+ *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
  * version 2 as published by the Free Software Foundation.
@@ -35,9 +35,7 @@
 #include <signal.h>
 
 #include "luks.h"
-//#include "../lib/libcryptsetup.h"
 #include "internal.h"
-//#include "../lib/blockdev.h"
 
 #define div_round_up(a,b) ({          \
 	typeof(a) __a = (a);          \
@@ -82,7 +80,7 @@ static void sigint_handler(int sig)
 	if(devfd >= 0)
 		close(devfd);
 	devfd = -1;
-	if(cleaner_name) 
+	if(cleaner_name)
 		dm_remove_device(cleaner_name, 1, cleaner_size);
 
 	signal(SIGINT, SIG_DFL);
@@ -107,23 +105,23 @@ static char *_error_hint(char *cipherName, char *cipherMode, size_t keyLength)
 	kernel_minor = atoi(tmp);
 
 	if (!strncmp(cipherMode, "xts", 3) && (keyLength != 256 && keyLength != 512))
-		hint = "Key size in XTS mode must be 256 or 512 bits.";
+		hint = _("Key size in XTS mode must be 256 or 512 bits.\n");
 	else if (!strncmp(cipherMode, "xts", 3) && kernel_minor < 24)
-		hint = "Block mode XTS is available since kernel 2.6.24.";
+		hint = _("Block mode XTS is available since kernel 2.6.24.\n");
 	if (!strncmp(cipherMode, "lrw", 3) && (keyLength != 256 && keyLength != 512))
-		hint = "Key size in LRW mode must be 256 or 512 bits.";
+		hint = _("Key size in LRW mode must be 256 or 512 bits.\n");
 	else if (!strncmp(cipherMode, "lrw", 3) && kernel_minor < 20)
-		hint = "Block mode LRW is available since kernel 2.6.20.";
+		hint = _("Block mode LRW is available since kernel 2.6.20.\n");
 #endif
 	return hint;
 }
 
 /* This function is not reentrant safe, as it installs a signal
    handler and global vars for cleaning */
-static int LUKS_endec_template(char *src, size_t srcLength, 
-			       struct luks_phdr *hdr, 
-			       char *key, size_t keyLength, 
-			       const char *device, 
+static int LUKS_endec_template(char *src, size_t srcLength,
+			       struct luks_phdr *hdr,
+			       char *key, size_t keyLength,
+			       const char *device,
 			       unsigned int sector,
 			       ssize_t (*func)(int, void *, size_t),
 			       int mode,
@@ -132,7 +130,7 @@ static int LUKS_endec_template(char *src, size_t srcLength,
 	char *name = NULL;
 	char *fullpath = NULL;
 	char *dmCipherSpec = NULL;
-	const char *dmDir = dm_get_dir(); 
+	const char *dmDir = dm_get_dir();
 	int r = -1;
 
 	if(dmDir == NULL) {
